"use strict"

const config = require('../config/config.json');
const MongoClient = require("../index.js");
const mongo = new MongoClient(config.url);
const assert = require('assert');

const Person = require('@josebarrios/person');
const Organization = require('@josebarrios/organization');
const Invoice = require('@josebarrios/invoice');

let tempID = null;

describe("Mongo API", function() {

  describe("Methods", function() {

    it("#insertDocument", function(done) {
      let data = {};
      data.givenName = 'Jose';
      data.familyName = 'Barrios';
      data.email = 'jose@barrios.io';
      data.additionalName = "Luis";
      let person = new Person(data);
      let doc = Person.assignedProperties(person, 'object');
      mongo.insertDocument('people', doc).then(res => {
        tempID = res._id;
        done()
      }).catch(err =>{ done(err) })
    })

    it("#getDocumentByID", function(done) {
      mongo.getDocumentByID('people', tempID)
        .then(res => {
          done()
        })
        .catch(err =>{
          done(err)
        })
    })


    it("#searchCollectionsForDocumentWithID", function(done) {
      mongo.searchCollectionsForDocumentWithID(['x','y','z','people'], tempID)
        .then(res => { done() })
        .catch(err =>{ done(err) })
    })

    it("#updateDocument", function(done) {
      mongo.getDocumentByID('people', tempID).then(personData => {
        let person = new Person(personData);
        let unset = Person.unsetProperties(person, 'object');
        let updates = {};
        updates.email = 'updated@email.com'
        mongo.updateDocument('people', tempID, updates, unset)
          .then(res => { done(); })
          .catch(done)
      }).catch(done)
    })

    it("#deleteDocument", function(done) {
      mongo.deleteDocument('people', tempID)
        .then(res => { done() })
        .catch(err =>{ done(err) })
    })

    it("#createCollectionWithUniqueIndices", function(done) {
      mongo.createCollectionWithUniqueIndices('users', {email: 1})
        .then(res => { done() })
        .catch(err =>{ done(err) })
    })

    it("#createCollectionWithTemporaryDocuments", function(done) {
      const delay = 10;
      const COLLECTION = 'tests.temp'
      const timestampKey = 'dateSent'
      mongo.createCollectionWithTemporaryDocuments(COLLECTION, timestampKey, delay)
        .then(res => {
          let document = {};
          document[timestampKey] = new Date();
          mongo.insertDocument(COLLECTION, document)
            .then(documentID => {
              done();
            }).catch(err =>{
              console.log('MAILER ERROR:', err)
              done(err)
            })
        }).catch(err =>{
          console.error('MAILER ERROR:', err)
          done(err)
        })
    })

    it("#done", function(done) {
      mongo.disconnect()
        .then(res => {
          assert(res, true)
          done();
        })
        .catch(done)
    })



  });
});
